//--------LICENSE_START--------
//
// Copyright (C) 2020,2021,2022 Institut de Robòtica i Informàtica Industrial, CSIC-UPC.
// Authors: Joan Solà Ortega (jsola@iri.upc.edu)
// All rights reserved.
//
// This file is part of WOLF
// WOLF is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//--------LICENSE_END--------
#include "gnss/capture/capture_gnss_fix.h"
#include <core/state_block/state_block_derived.h>


namespace wolf {

CaptureGnssFix::CaptureGnssFix(const TimeStamp& _ts, 
							   SensorBasePtr _sensor_ptr, 
							   const Eigen::Vector4d& _fix, 
							   const Eigen::Matrix4d& _covariance, 
							   bool _ecef_coordinates) :
	CaptureBase("CaptureGnssFix", _ts, _sensor_ptr),
	fix_ECEF_(_fix),
	covariance_ECEF_(_covariance)
{
	// Convert to ECEF from latlonalt
	if (not _ecef_coordinates)
	{
		fix_ECEF_.head<3>() = GnssUtils::latLonAltToEcef(_fix.head<3>());
		covariance_ECEF_.topLeftCorner<3,3>() = GnssUtils::enuToEcefCov(_fix.head<3>(), 
																	    _covariance.topLeftCorner<3,3>());
	}

    // Clock bias
    assert(_sensor_ptr->getStateBlock('T') != nullptr and _sensor_ptr->isStateBlockDynamic('T'));
    addStateBlock('T', std::make_shared<StateParams1>(Vector1d(),true), nullptr);

    // interconstellation clock bias
    assert(_sensor_ptr->getStateBlock('G') != nullptr);
    if(_sensor_ptr->isStateBlockDynamic('G'))
        addStateBlock('G', std::make_shared<StateParams1>(Vector1d(),true), nullptr);
    assert(_sensor_ptr->getStateBlock('E') != nullptr);
    if(_sensor_ptr->isStateBlockDynamic('E'))
        addStateBlock('E', std::make_shared<StateParams1>(Vector1d(),true), nullptr);
    assert(_sensor_ptr->getStateBlock('M') != nullptr);
    if(_sensor_ptr->isStateBlockDynamic('M'))
        addStateBlock('M', std::make_shared<StateParams1>(Vector1d(),true), nullptr);
}

CaptureGnssFix::~CaptureGnssFix()
{
	//std::cout << "Destroying GPS fix capture...\n";
}

} //namespace wolf
